import { readFile, writeFile } from 'node:fs/promises';
import { readFileSync, writeFileSync } from 'node:fs';

export class lazyKv {
  path: string;

  constructor(_path: string) {
    this.path = _path;
    this.init();
  }

  init(): void {
    try {
      readFileSync(this.path, 'utf-8');
    } catch {
      writeFileSync(this.path, '{}', 'utf-8');
    } finally {
      console.log(`LazyKv: initialized ${this.path}`);
    }
  }

  async get<T>(key: string, fallback?: T): Promise<T> {
    const data = await readFile(this.path, 'utf-8');
    const json = JSON.parse(data);

    return json[key] ?? (fallback || null);
  }

  async set<T>(key: string, value: T): Promise<void> {
    const data = await readFile(this.path, 'utf-8');
    const json = JSON.parse(data);

    json[key] = value;

    await writeFile(this.path, JSON.stringify(json, null, 2), 'utf-8');
  }

  async delete(key: string): Promise<void> {
    const data = await readFile(this.path, 'utf-8');
    const json = JSON.parse(data);

    delete json[key];

    await writeFile(this.path, JSON.stringify(json, null, 2), 'utf-8');
  }
}
